clear all
close all
clc

mData = xlsread('DataWeakIVpm.xls');

mData = mData(1:85,:);    % SELECT ONLY DATA UNTIL 1990 

% variables
vPi    = mData(:,3);
vPiA   = mData(:,3) - mData(:,12); % expectation adjusted PI 
vPiE   = mData(:,12);              % expectation 
vU     = mData(:,11);
mZ     = mData(:,8:10);

% set control variables
mX    = [lagmatrix(vPiA,1) lagmatrix(vPiA,2) lagmatrix(vPiA,3) lagmatrix(vPiA,4) lagmatrix(vU,1) lagmatrix(vU,2) lagmatrix(vU,3) lagmatrix(vU,4)];
mX2   = [lagmatrix(vPi,1) lagmatrix(vPi,2) lagmatrix(vPi,3) lagmatrix(vPi,4) lagmatrix(vU,1) lagmatrix(vU,2) lagmatrix(vU,3) lagmatrix(vU,4)];

% run cummulative regressions 
iH       = 21;

vBeta     = zeros(iH,1);             % storage 2SLS point 
vBetaE    = zeros(iH,1);             % storage 2SLS point 
vLowerAR  = zeros(iH,1);             % lower AR-bound  
vUpperAR  = zeros(iH,1);             % upper AR-bound 
vLowerARE = zeros(iH,1);             % lower AR-bound  
vUpperARE = zeros(iH,1);             % upper AR-bound 
vLowerAR2s  = zeros(iH,1);             % lower AR-bound  
vUpperAR2s  = zeros(iH,1);             % upper AR-bound 
vLowerARE2s = zeros(iH,1);             % lower AR-bound  
vUpperARE2s = zeros(iH,1);             % upper AR-bound 
vFstat    = zeros(iH,1);             % F-stats 
vParB     = (-10:0.01:5)';           % parameters to test 
mWald     = zeros(size(vParB,1),iH); % store Wald
mTest     = zeros(size(vParB,1),iH); % store rejection 

for h=1:iH
    vPiC  = vPi;    % inlfation 
    vPiEC = vPiE;   % expected inflation
    vPiAC = vPiA;   % inflation adjusted for expectations
    vUC   = vU ;    % unemployment 
    if h>1                                       % construct cumulative pi, u, pi^e, pi - pi^e 
        for j=2:h
            vPiC  = vPiC + lagmatrix(vPi,-(j-1));
            vUC   = vUC  + lagmatrix(vU,-(j-1));
            vPiEC = vPiEC + lagmatrix(vPiE,-(j-1));
            vPiAC = vPiAC + lagmatrix(vPiA,-(j-1));            
        end
    end 
   
    % COMPUTE THE PM  
    mObs      = [vPiC vUC mZ mX2];                                % drop the missing observations 
    mObs      = rmmissing(mObs);
    
    iTc       = size(mObs,1);                                     % observations 
   
    vPiCu     = mObs(:,1);
    vUCu      = mObs(:,2);
    mZCu      = sum(mObs(:,3:5),2);
    mXCu      = [ones(iTc,1) mObs(:,6:end)];                        % controls PM     
    
    iL       = size(mZCu , 2);                  % number of instruments   
       
    mMx      = eye(iTc) - mXCu * inv(mXCu'*mXCu) * mXCu';  % project out constant and control variables   
    
    vPiCu    = mMx * vPiCu;
    vUCu     = mMx * vUCu;    
    
    % IV reg 
    vCoefs    = inv(vUCu' * mZCu * inv(mZCu'*mZCu) * mZCu' * vUCu) * vUCu' * mZCu * inv(mZCu'*mZCu) * mZCu' * vPiCu;  %  TSLS(vPiCu,vUCu,mZCu,0,0); 
    vBeta(h)  = vCoefs(1);                      % 2SLS coefs 
    
     % standard bounds 
    vRes   = vPiCu -  vBeta(h) * vUCu;
    mV     = NeweyWest(vRes, mZCu ,-1,0);
    mPz    = mZCu * inv(mZCu' * mZCu) * mZCu';
    mAvar  = inv(vUCu' * mPz * vUCu) * vUCu' * mZCu * inv(mZCu' * mZCu) * mV * inv(mZCu' * mZCu) * mZCu' * vUCu * inv(vUCu' * mPz * vUCu);
    vLowerAR2s(h) = vBeta(h) - 1.96 * sqrt(mAvar);
    vUpperAR2s(h) = vBeta(h) + 1.96 * sqrt(mAvar);
    
    vTest     = zeros(size(vParB,1),1);
    for k = 1:size(vParB,1) 
        vRes   = vPiCu - vParB(k) * vUCu;                       % residual 
        vGamma = mZCu' * vRes;       
        vRes1  = vRes - mZCu *  inv(mZCu'*mZCu) *  vGamma;       
        mV     = NeweyWest(vRes1, mZCu ,4,0);                  % dimension 2L x 2L 
        
        dWald  = vGamma' * inv(mV) * vGamma;               % AR statistic  
      
        if (dWald < chi2inv(0.95,iL))
            vTest(k) = 1; 
        end
        
    end

     % find upper and lower confidence bounds AR statistic  
    vBounds = FindUpperLower(vTest, vParB);
    vLowerAR(h) = vBounds(1);
    vUpperAR(h) = vBounds(2);    
   
    % compute effective F-stat 
    vFstat(h)  = EffectiveFstat(vUCu, mZCu); 
    
    % COMPUTE THE PM - K  
    mObs      = [vPiAC vUC mZ mX];                                % drop the missing observations 
    mObs      = rmmissing(mObs);
    
    iTc       = size(mObs,1);                                     % observations 
   
    vPiACu    = mObs(:,1);
    vUCu      = mObs(:,2);
    mZCu      = sum(mObs(:,3:5),2);
    mXCu      = [ones(iTc,1) mObs(:,6:13)];                        % controls PM     
    
    iL       = size(mZCu , 2);                  % number of instruments   
       
    mMx      = eye(iTc) - mXCu * inv(mXCu'*mXCu) * mXCu';  % project out constant and control variables   
    
    vPiACu   = mMx * vPiACu;
    vUCu     = mMx * vUCu;    

    % IV reg 
    vCoefs   = inv(vUCu' * mZCu * inv(mZCu'*mZCu) * mZCu' * vUCu) * vUCu' * mZCu * inv(mZCu'*mZCu) * mZCu' * vPiACu;   %  TSLS(vPiACu,vUCu,mZCu,0,0); 
    vBetaE(h) = vCoefs(1);                      % 2SLS coefs 
    
    vRes   = vPiACu -  vBetaE(h) * vUCu;
    mV     = NeweyWest(vRes, mZCu ,-1,0);
    mPz    = mZCu * inv(mZCu' * mZCu) * mZCu';
    mAvar  = inv(vUCu' * mPz * vUCu) * vUCu' * mZCu * inv(mZCu' * mZCu) * mV * inv(mZCu' * mZCu) * mZCu' * vUCu * inv(vUCu' * mPz * vUCu);
    vLowerARE2s(h) = vBetaE(h) - 1.96 * sqrt(mAvar);
    vUpperARE2s(h) = vBetaE(h) + 1.96 * sqrt(mAvar);
    
    vTest     = zeros(size(vParB,1),1);
    for k = 1:size(vParB,1) 
        vRes   = vPiACu - vParB(k) * vUCu;                       % residual 
        vGamma = mZCu' * vRes;       
        vRes1  = vRes - mZCu *  inv(mZCu'*mZCu) *  vGamma;       
        mV     = NeweyWest(vRes1, mZCu ,4,0);                  % dimension 2L x 2L 
        
        dWald  = vGamma' * inv(mV) * vGamma;               % AR statistic  
      
        if (dWald < chi2inv(0.95,iL))
            vTest(k) = 1; 
        end
        
    end

     % find upper and lower confidence bounds AR statistic  
    vBounds = FindUpperLower(vTest, vParB);
    vLowerARE(h) = vBounds(1);
    vUpperARE(h) = vBounds(2);    
   
    % compute effective F-stat 
    vFstat(h)  = EffectiveFstat(vUCu, mZCu);    
    
    
    % COMPUTE IMPLIED A 
    vA = 1 - vBetaE ./ vBeta;
end

mRomerRomer = [ vLowerAR vBeta vUpperAR vLowerARE vBetaE vUpperARE vA vLowerAR2s vUpperAR2s vLowerARE2s vUpperARE2s]; 


mData = xlsread('DataWeakIVhffinal.xlsx');
 

% variables
vPi   = mData(:,6);
vPiA  = mData(:,6) - mData(:,12); % inflation expectation adjusted 
vPiE  = mData(:,12);
vU    = mData(:,3);
mZ    = [mData(:,28:30) mData(:,28:30).^2];
vFFR  = mData(:,17);
mXF   = mData(:,31:39);


% set control variables
mX    = [lagmatrix(vPiA,1) lagmatrix(vPiA,2) lagmatrix(vPiA,3) lagmatrix(vPiA,4) lagmatrix(vU,1) lagmatrix(vU,2) lagmatrix(vU,3) lagmatrix(vU,4) ];
mX2   = [lagmatrix(vPi,1) lagmatrix(vPi,2) lagmatrix(vPi,3) lagmatrix(vPi,4) lagmatrix(vU,1) lagmatrix(vU,2) lagmatrix(vU,3) lagmatrix(vU,4)];

% run cummulative regressions 
iH       = 21;
iM       = 10000;

vBeta    = zeros(iH,1);             % storage 2SLS point 
vBetaPi  = zeros(iH,1);             % storage irf pi   
vBetaUr  = zeros(iH,1);             % storage irf ur        
vLowerAR = zeros(iH,1);             % lower AR-bound  
vUpperAR = zeros(iH,1);             % upper AR-bound 
vLowerARE= zeros(iH,1);             % lower AR-bound  
vUpperARE= zeros(iH,1);             % upper AR-bound 
vFstat   = zeros(iH,1);             % F-stats 
vParB    = (-10:0.01:5)';           % parameters to test 
mWald    = zeros(size(vParB,1),iH); % store Wald
mTest    = zeros(size(vParB,1),iH); % store rejection 

for h=1:iH
    vPiC = vPi;  
    vPiEC= vPiE;
    vPiAC= vPiA;
    vUC  = vU ;
    if h>1                                       % construct cumulative pi and u 
        for j=2:h
            vPiC  = vPiC + lagmatrix(vPi,-(j-1));
            vPiAC = vPiAC + lagmatrix(vPiA,-(j-1)); 
            vPiEC = vPiEC + lagmatrix(vPiE,-(j-1)); 
            vUC   = vUC  + lagmatrix(vU,-(j-1));
        end
    end 
    
    % COMPUTE PM 
    
    mObs     = [vPiC vUC mZ mX2];                % drop the missing observations 
    mObs     = rmmissing(mObs);
    
    iTc      = size(mObs,1);                                     % observations 
    
    vPiCu    = mObs(:,1);
    vUCu     = mObs(:,2);
    mZCu     = sum(mObs(:,3:5),2);
    mXCu     = [ones(iTc,1) mObs(:,9:end)];                        % controls PM     
    
    iL       = size(mZCu , 2);                  % number of instruments   
    
    mMx      = eye(iTc) - mXCu * inv(mXCu'*mXCu) * mXCu';  % project out constant and control variables   
    
    vPiCu     = mMx * vPiCu;
    vUCu      = mMx * vUCu;    

    % IV reg 
    vCoefs   = inv(vUCu' * mZCu * inv(mZCu'*mZCu) * mZCu' * vUCu) * vUCu' * mZCu * inv(mZCu'*mZCu) * mZCu' * vPiCu; 
    vBeta(h) = vCoefs(1);                      % 2SLS coefs 
    
    vRes   = vPiCu -  vBeta(h) * vUCu;
    mV     = NeweyWest(vRes, mZCu ,12,0);
    mPz    = mZCu * inv(mZCu' * mZCu) * mZCu';
    mAvar  = inv(vUCu' * mPz * vUCu) * vUCu' * mZCu * inv(mZCu' * mZCu) * mV * inv(mZCu' * mZCu) * mZCu' * vUCu * inv(vUCu' * mPz * vUCu);
    vLowerAR2s(h) = vBeta(h) - 1.96 * sqrt(mAvar);
    vUpperAR2s(h) = vBeta(h) + 1.96 * sqrt(mAvar);
    
    vTest    = zeros(size(vParB,1),1);  
    for k = 1:size(vParB,1) 
        vRes   = vPiCu - vParB(k) * vUCu;                       % residual 
        vGamma = mZCu' * vRes;       
        vRes1  = vRes - mZCu *  inv(mZCu'*mZCu) *  vGamma;       
        mV     = NeweyWest(vRes1, mZCu ,4,0);                  % dimension 2L x 2L 
        
        dWald  = vGamma' * inv(mV) * vGamma;               % AR statistic  
      
        if (dWald < chi2inv(0.95,iL))
            vTest(k) = 1; 
        end
        
    end

     % find upper and lower confidence bounds AR statistic  
    vBounds = FindUpperLower(vTest, vParB);
    vLowerAR(h) = vBounds(1);
    vUpperAR(h) = vBounds(2);    
   
    % compute effective F-stat 
    vFstat(h)  = EffectiveFstat(vUCu, mZCu);     
    
    
    % COMPUTE PM-K 

    mObs     = [vPiAC vUC mZ mX2];                % drop the missing observations 
    mObs     = rmmissing(mObs);
    
    iTc      = size(mObs,1);                                     % observations 
    
    vPiACu   = mObs(:,1);
    vUCu     = mObs(:,2);
    mZCu     = sum(mObs(:,3:5),2) ;
    mXCu     = [ones(iTc,1) mObs(:,9:end)];                        % controls PM     
    
    iL       = size(mZCu , 2);                  % number of instruments   
    
    mMx      = eye(iTc) - mXCu * inv(mXCu'*mXCu) * mXCu';  % project out constant and control variables   
    
    vPiACu   = mMx * vPiACu;
    vUCu     = mMx * vUCu;        

     % IV reg 
    vCoefs    =  inv(vUCu' * mZCu * inv(mZCu'*mZCu) * mZCu' * vUCu) * vUCu' * mZCu * inv(mZCu'*mZCu) * mZCu' * vPiACu; 
    vBetaE(h) = vCoefs(1);                      % 2SLS coefs 
 
    vRes   = vPiACu -  vBetaE(h) * vUCu;
    mV     = NeweyWest(vRes, mZCu ,12,0);
    mPz    = mZCu * inv(mZCu' * mZCu) * mZCu';
    mAvar  = inv(vUCu' * mPz * vUCu) * vUCu' * mZCu * inv(mZCu' * mZCu) * mV * inv(mZCu' * mZCu) * mZCu' * vUCu * inv(vUCu' * mPz * vUCu);
    vLowerARE2s(h) = vBetaE(h) - 1.96 * sqrt(mAvar);
    vUpperARE2s(h) = vBetaE(h) + 1.96 * sqrt(mAvar);
    
    
    vTest     = zeros(size(vParB,1),1);   
    for k = 1:size(vParB,1) 
        vRes   = vPiACu - vParB(k) * vUCu;                       % residual 
        vGamma = mZCu' * vRes;       
        vRes1  = vRes - mZCu *  inv(mZCu'*mZCu) *  vGamma;       
        mV     = NeweyWest(vRes1, mZCu ,4,0);                  % dimension 2L x 2L 
        
        dWald  = vGamma' * inv(mV) * vGamma;               % AR statistic  
      
        if (dWald < chi2inv(0.95,iL))
            vTest(k) = 1; 
        end
       
    end

     % find upper and lower confidence bounds AR statistic  
    vBounds = FindUpperLower(vTest, vParB);
    vLowerARE(h) = vBounds(1);
    vUpperARE(h) = vBounds(2);    
   
    % compute effective F-stat 
    vFstat(h)  = EffectiveFstat(vUCu, mZCu);     
    
    
    vA = 1  - vBetaE ./ vBeta;

end

mHFI = [ vLowerAR vBeta vUpperAR vLowerARE vBetaE vUpperARE vA vLowerAR2s vUpperAR2s vLowerARE2s vUpperARE2s]; 


% MAKE FIGURE 

kappah= [NaN(5,1) ; mRomerRomer(6:end,2)];
kappahub=mRomerRomer(:,3);
kappahlb=mRomerRomer(:,1);

H=iH;

error1 = abs(mRomerRomer(:,2) - kappahlb);
error2 = abs(mRomerRomer(:,2) - kappahub);

x=(1:length(kappah))';

figure, 
sbp2=subplot(221), hold on
plot(x,0*kappah ,'LineWidth',1,'Color',[.7 .7 .7],'LineStyle','--');
H1= shadedErrorBar(x,mRomerRomer(:,2),[error2 error1], 'k');
g(1)=plot(kappah,'Linewidth',2,'Color','k','LineStyle','-');
g(1)=plot(mRomerRomer(:,8),'Linewidth',1,'Color','k','LineStyle',':');
g(1)=plot(mRomerRomer(:,9),'Linewidth',1,'Color','k','LineStyle',':');
ylabel('$\mathcal P_h$','Interpreter','latex','Fontsize',14)
ylim([-5 1])
xlim([1 H]);
set(gca','Xticklabel',num2str((0:5:H+1)'))
set(gca,'Xtick',1:5:H+1), xlim([1 H]),
set(gca,'Ytick',-14:2:14), xlim([1 H]),
ax=get(sbp2,'Position');
set(sbp2,'Position',ax);
title('Pre-1990, RR','Interpreter','latex')

kappah=[NaN(5,1) ; mRomerRomer(6:end,5)];
kappahub=mRomerRomer(:,6);
kappahlb=mRomerRomer(:,4);

H=iH;

error1 = abs(mRomerRomer(:,5) - kappahlb);
error2 = abs(mRomerRomer(:,5) - kappahub);

x=(1:length(kappah))';


sbp2=subplot(223), hold on
plot(x,0*kappah ,'LineWidth',1,'Color',[.7 .7 .7],'LineStyle','--');
H1= shadedErrorBar(x,mRomerRomer(:,5),[error2 error1], 'k');
g(1)=plot(kappah,'Linewidth',2,'Color','k','LineStyle','-');
g(1)=plot(mRomerRomer(:,10),'Linewidth',1,'Color','k','LineStyle',':');
g(1)=plot(mRomerRomer(:,11),'Linewidth',1,'Color','k','LineStyle',':');
ylabel('$\mathcal K_h$','Interpreter','latex','Fontsize',14)
ylim([-5 1])
xlim([1 H]);
set(gca','Xticklabel',num2str((0:5:H+1)'))
set(gca,'Xtick',1:5:H+1), xlim([1 H]),
set(gca,'Ytick',-14:2:14), xlim([1 H]),
ax=get(sbp2,'Position');
set(sbp2,'Position',ax);
title(' ','Interpreter','latex')



%%%%%%%%%%%%%%%%%
% 2) Plot Pm from HFI post 1990 (pre-ZLB)
%%%%%%%%%%%%%%%%%


kappah= [NaN(5,1) ; mHFI(6:end,2)];
kappahub= mHFI(:,3);
kappahlb= mHFI(:,1);

H=iH;

error1 = abs(mHFI(:,2) - kappahlb);
error2 = abs(mHFI(:,2) - kappahub);

x=(1:length(kappah))';

sbp2=subplot(222), hold on
plot(x,0*kappah ,'LineWidth',1,'Color',[.7 .7 .7],'LineStyle','--');
H1= shadedErrorBar(x,mHFI(:,2),[error2 error1], 'k');
g(1)=plot([kappah(1:end)],'Linewidth',2,'Color','k','LineStyle','-');
g(1)=plot(mHFI(:,8),'Linewidth',1,'Color','k','LineStyle',':');
g(1)=plot(mHFI(:,9),'Linewidth',1,'Color','k','LineStyle',':');
ylim([-0.8 0.4])
xlim([1 H]);
set(gca','Xticklabel',num2str((0:5:H+1)'))
set(gca,'Xtick',1:5:H+1), xlim([1 H]),
set(gca,'Ytick',-14:.2:14), xlim([1 H]),

ax=get(sbp2,'Position');
set(sbp2,'Position',ax);
title('Post-1990, HFI','Interpreter','latex');


kappah= [NaN(5,1) ; mHFI(6:end,5)];
kappahub= mHFI(:,6);
kappahlb= mHFI(:,4);

H=iH;

error1 = abs(mHFI(:,5) - kappahlb);
error2 = abs(mHFI(:,5) - kappahub);

x=(1:length(kappah))';


sbp2=subplot(224), hold on
plot(x,0*kappah ,'LineWidth',1,'Color',[.7 .7 .7],'LineStyle','--');
H1= shadedErrorBar(x,mHFI(:,5),[error2 error1], 'k');
g(1)=plot([kappah(1:end)],'Linewidth',2,'Color','k','LineStyle','-');
g(1)=plot(mHFI(:,10),'Linewidth',1,'Color','k','LineStyle',':');
g(1)=plot(mHFI(:,11),'Linewidth',1,'Color','k','LineStyle',':');
ylim([-0.8 0.4])
xlim([1 H]);
set(gca','Xticklabel',num2str((0:5:H+1)'))
set(gca,'Xtick',1:5:H+1), xlim([1 H]),
set(gca,'Ytick',-14:.2:14), xlim([1 H]),

ax=get(sbp2,'Position');
set(sbp2,'Position',ax);
title('','Interpreter','latex');



