clear all
close all
clc

mData = xlsread('DataWeakIVpm.xls');

mData = mData(1:85,:);    % SELECT ONLY DATA UNTIL 1990 

% variables
vPi    = mData(:,3);
vPiA   = mData(:,3) - mData(:,12); % expectation adjusted PI 
vPiE   = mData(:,12);              % expectation 
vU     = mData(:,11);
mZ     = mData(:,8:10);

% set control variables
mX    = [lagmatrix(vPi,1) lagmatrix(vPi,2) lagmatrix(vPi,3) lagmatrix(vPi,4) lagmatrix(vU,1) lagmatrix(vU,2) lagmatrix(vU,3) lagmatrix(vU,4)];

% run cummulative regressions 
iH       = 21;

vBeta    = zeros(iH,1);             % storage 2SLS point 
vBetaE  = zeros(iH,1);              % storage irf E       
vBetaU  = zeros(iH,1);              % storage irf U  
vLower2s = zeros(iH,1);             % lower AR-bound  
vUpper2s = zeros(iH,1);             % upper AR-bound 
vLowerE2s= zeros(iH,1);             % lower AR-bound  
vUpperE2s= zeros(iH,1);             % upper AR-bound 
vLowerU2s= zeros(iH,1);             % lower AR-bound  
vUpperU2s= zeros(iH,1);             % upper AR-bound 


for h=1:iH
    vPiC = vPi;  
    vPiEC= vPiE;
    vPiAC= vPiA;
    vUC  = vU ;
    if h>1                                       % construct cumulative pi and u 
        for j=2:h
            vPiC  = vPiC + lagmatrix(vPi,-(j-1));
            vPiAC = vPiAC + lagmatrix(vPiA,-(j-1)); 
            vPiEC = vPiEC + lagmatrix(vPiE,-(j-1)); 
            vUC   = vUC  + lagmatrix(vU,-(j-1));
        end
    end 
    
    % IRF's 
    
    mObs     = [vPiC vPiEC vUC mZ mX];                % drop the missing observations 
    mObs     = rmmissing(mObs);
    
    iTc      = size(mObs,1);                                     % observations 
    
    vPiCu    = mObs(:,1);
    vPiECu   = mObs(:,2); 
    vUCu     = mObs(:,3);
    mZCu     = sum(mObs(:,4:6),2);
    mXCu     = [ones(iTc,1) mObs(:,7:end)];                        % controls PM     
    
    iL       = size(mZCu , 2);                  % number of instruments   
    
    mMx      = eye(iTc) - mXCu * inv(mXCu'*mXCu) * mXCu';  % project out constant and control variables   
    
    vPiCu     = mMx * vPiCu;
    vPiECu    = mMx * vPiECu;  
    vUCu    = mMx * vUCu; 

    % IV reg 
    vBeta(h) = inv(mZCu'*mZCu) * mZCu' * vPiCu;                      % 2SLS coefs 
    
    vRes   = vPiCu -  vBeta(h) * mZCu;
    mV     = NeweyWest(vRes, mZCu ,4,0);
    mAvar  = inv(mZCu' * mZCu) * mV * inv(mZCu' * mZCu);
    vLower2s(h) = vBeta(h) - 1.96 * sqrt(mAvar);
    vUpper2s(h) = vBeta(h) + 1.96 * sqrt(mAvar);
    
    vBetaE(h) = inv(mZCu'*mZCu) * mZCu' * vPiECu;  
    
    vRes   = vPiECu -  vBetaE(h) * mZCu;
    mV     = NeweyWest(vRes, mZCu ,4,0);
    mAvar  = inv(mZCu' * mZCu) * mV * inv(mZCu' * mZCu);
    vLowerE2s(h) = vBetaE(h) - 1.96 * sqrt(mAvar);
    vUpperE2s(h) = vBetaE(h) + 1.96 * sqrt(mAvar);
    
    vBetaU(h) = inv(mZCu'*mZCu) * mZCu' * vUCu;  
    
    vRes   = vUCu -  vBetaU(h) * mZCu;
    mV     = NeweyWest(vRes, mZCu ,4,0);
    mAvar  = inv(mZCu' * mZCu) * mV * inv(mZCu' * mZCu);
    vLowerU2s(h) = vBetaU(h) - 1.96 * sqrt(mAvar);
    vUpperU2s(h) = vBetaU(h) + 1.96 * sqrt(mAvar);
    

end

mRomerRomer = [vLower2s vBeta vUpper2s vLowerE2s vBetaE vUpperE2s vLowerU2s vBetaU vUpperU2s]; 


mData = xlsread('DataWeakIVhffinal.xlsx');
 

% variables
vPi   = mData(:,6);
vPiA  = mData(:,6) - mData(:,12); % inflation expectation adjusted 
vPiE  = mData(:,12);
vU    = mData(:,3);
mZ    = mData(:,28:30);
vFFR  = mData(:,17);
mXF   = mData(:,31:39);


% set control variables
mX    = [lagmatrix(vPi,1) lagmatrix(vPi,2) lagmatrix(vPi,3) lagmatrix(vPi,4) lagmatrix(vU,1) lagmatrix(vU,2) lagmatrix(vU,3) lagmatrix(vU,4)];

% run cummulative regressions 
iH       = 21;

vBeta    = zeros(iH,1);             % storage 2SLS point 
vBetaE  = zeros(iH,1);              % storage irf E       
vLower2s = zeros(iH,1);             % lower AR-bound  
vUpper2s = zeros(iH,1);             % upper AR-bound 
vLowerE2s= zeros(iH,1);             % lower AR-bound  
vUpperE2s= zeros(iH,1);             % upper AR-bound 

for h=1:iH
    vPiC = vPi;  
    vPiEC= vPiE;
    vPiAC= vPiA;
    vUC  = vU ;
    if h>1                                       % construct cumulative pi and u 
        for j=2:h
            vPiC  = vPiC + lagmatrix(vPi,-(j-1));
            vPiAC = vPiAC + lagmatrix(vPiA,-(j-1)); 
            vPiEC = vPiEC + lagmatrix(vPiE,-(j-1)); 
            vUC   = vUC  + lagmatrix(vU,-(j-1));
        end
    end 
    
    % IRF's 
    
    mObs     = [vPiC vPiEC vUC mZ mX];                % drop the missing observations 
    mObs     = rmmissing(mObs);
    
    iTc      = size(mObs,1);                                     % observations 
    
    vPiCu    = mObs(:,1);
    vPiECu   = mObs(:,2); 
    vUCu     = mObs(:,3);
    mZCu     = sum(mObs(:,4:6),2);
    mXCu     = [ones(iTc,1) mObs(:,7:end)];                        % controls PM     
    
    iL       = size(mZCu , 2);                  % number of instruments   
    
    mMx      = eye(iTc) - mXCu * inv(mXCu'*mXCu) * mXCu';  % project out constant and control variables   
    
    vPiCu     = mMx * vPiCu;
    vPiECu    = mMx * vPiECu;  
    vUCu    = mMx * vUCu; 

    % IV reg 
    vBeta(h) = inv(mZCu'*mZCu) * mZCu' * vPiCu;                      % 2SLS coefs 
    
    vRes   = vPiCu -  vBeta(h) * mZCu;
    mV     = NeweyWest(vRes, mZCu ,12,0);
    mAvar  = inv(mZCu' * mZCu) * mV * inv(mZCu' * mZCu);
    vLower2s(h) = vBeta(h) - 1.96 * sqrt(mAvar);
    vUpper2s(h) = vBeta(h) + 1.96 * sqrt(mAvar);
    
    vBetaE(h) = inv(mZCu'*mZCu) * mZCu' * vPiECu;  
    
    vRes   = vPiECu -  vBetaE(h) * mZCu;
    mV     = NeweyWest(vRes, mZCu ,12,0);
    mAvar  = inv(mZCu' * mZCu) * mV * inv(mZCu' * mZCu);
    vLowerE2s(h) = vBetaE(h) - 1.96 * sqrt(mAvar);
    vUpperE2s(h) = vBetaE(h) + 1.96 * sqrt(mAvar);
    
    vBetaU(h) = inv(mZCu'*mZCu) * mZCu' * vUCu;  
    
    vRes   = vUCu -  vBetaU(h) * mZCu;
    mV     = NeweyWest(vRes, mZCu ,12,0);
    mAvar  = inv(mZCu' * mZCu) * mV * inv(mZCu' * mZCu);
    vLowerU2s(h) = vBetaU(h) - 1.96 * sqrt(mAvar);
    vUpperU2s(h) = vBetaU(h) + 1.96 * sqrt(mAvar);
    

end

mHFI = [ vLower2s vBeta vUpper2s vLowerE2s vBetaE vUpperE2s vLowerU2s vBetaU vUpperU2s]; 



% PLOT IRF's 

%%%%%%%%%%%%%%%%%
% 1) Plot IRFs from  pre 1990
%%%%%%%%%%%%%%%%%

%rescale IRFs so that ir of ur peaks at 1:
irPi       = mRomerRomer(:,[1 2 3]) / max(mRomerRomer(:,8));
irPiE      = mRomerRomer(:,[4 5 6]) / max(mRomerRomer(:,8));

vA         = mRomerRomer(:,5) ./ mRomerRomer(:,2);

irPi_hfi   = mHFI(:,[1 2 3]) / max(mHFI(:,8));
irPiE_hfi  = mHFI(:,[4 5 6]) / max(mHFI(:,8));

vA1         = mHFI(:,5) ./ mHFI(:,2);

H=length(irPi);
x=(1:H)';


figure(1),

subplot(2,2,1), hold on
plot(x,0*irPi ,'LineWidth',1,'Color',[.7 .7 .7],'LineStyle','--');
H1= shadedErrorBar(x,irPi(:,2),[abs(irPi(:,2) - irPi(:,3)) abs(irPi(:,2) - irPi(:,1))], 'k');
g(1)=plot(irPi(:,2),'Linewidth',2,'Color','k','LineStyle','-');
ylabel('{$\mathcal R^{\bar{\pi}}_h$}','Interpreter','latex','Fontsize',14);
ylim([-5 2])
set(gca,'Ytick',-10:2:10)
xlim([1 H]);
set(gca','Xticklabel',num2str((0:5:H+1)'))
set(gca,'Xtick',1:5:H+1), xlim([1 H]),
title('Pre-1990, RR','Interpreter','latex')  
    
subplot(2,2,3), hold on
plot(x,0*irPiE ,'LineWidth',1,'Color',[.7 .7 .7],'LineStyle','--');
H1= shadedErrorBar(x,irPiE(:,2),[abs(irPiE(:,2) - irPiE(:,3)) abs(irPiE(:,2) - irPiE(:,1))], 'k');
g(1)=plot(irPiE(:,2),'Linewidth',2,'Color','k','LineStyle','-');
ylabel('{$\mathcal R^{\bar{\pi^e}}_h$}','Interpreter','latex','Fontsize',14);
ylim([-5 2])
set(gca,'Ytick',-10:2:10)
xlim([1 H]);
set(gca','Xticklabel',num2str((0:5:H+1)'))
set(gca,'Xtick',1:5:H+1), xlim([1 H]),

subplot(2,2,2), hold on
plot(x,0*irPi_hfi ,'LineWidth',1,'Color',[.7 .7 .7],'LineStyle','--');
H1= shadedErrorBar(x,irPi_hfi(:,2),[abs(irPi_hfi(:,2) - irPi_hfi(:,3)) abs(irPi_hfi(:,2) - irPi_hfi(:,1))], 'k');
g(1)=plot(irPi_hfi(:,2),'Linewidth',2,'Color','k','LineStyle','-');
%ylabel('{$\mathcal R^{\bar{\pi}}_h$}','Interpreter','latex','Fontsize',14);
ylim([-0.2 0.2])
set(gca,'Ytick',-10:.1:10)
xlim([1 H]);
set(gca','Xticklabel',num2str((0:5:H+1)'))
set(gca,'Xtick',1:5:H+1), xlim([1 H]),
title('Post-1990, HFI','Interpreter','latex')  
    
subplot(2,2,4), hold on
plot(x,0*irPiE_hfi ,'LineWidth',1,'Color',[.7 .7 .7],'LineStyle','--');
H1= shadedErrorBar(x,irPiE_hfi(:,2),[abs(irPiE_hfi(:,2) - irPiE_hfi(:,3)) abs(irPiE_hfi(:,2) - irPiE_hfi(:,1))], 'k');
g(1)=plot(irPiE_hfi(:,2),'Linewidth',2,'Color','k','LineStyle','-');
%ylabel('{$\mathcal R^{\bar{\pi}^e}_h$}','Interpreter','latex','Fontsize',14);
ylim([-0.2 0.2])
xlim([1 H]);
set(gca','Xticklabel',num2str((0:5:H+1)'))
set(gca,'Xtick',1:5:H+1), xlim([1 H]),
set(gca,'Ytick',-10:.1:10)


