clear all
close all
clc


% %  % SET UP Romer Romer part 
mData = xlsread('DataWeakIVpm.xls');
mData = mData(1:80,:);    % SELECT ONLY DATA UNTIL 1990 
vPi   = mData(:,3);
vU    = mData(:,11);
mZ    = sum(mData(:,8:10),2);
mX    = [lagmatrix(vPi,1) lagmatrix(vPi,2) lagmatrix(vPi,3) lagmatrix(vPi,4) lagmatrix(vU,1) lagmatrix(vU,2) lagmatrix(vU,3) lagmatrix(vU,4) ];
iH       = 21;

% Storage results  
vBeta    = zeros(iH,1);             % storage 2SLS point 
vBetaPi  = zeros(iH,1);             % storage irf pi   
vBetaUr  = zeros(iH,1);             % storage irf ur        
vLowerAR = zeros(iH,1);             % lower AR-bound  
vUpperAR = zeros(iH,1);             % upper AR-bound 
vLowerAR2s= zeros(iH,1);            % lower 2SLS-bound  
vUpperAR2s= zeros(iH,1);            % upper 2SLS-bound  
vLowerPI = zeros(iH,1);             % lower bound AR pi 
vUpperPI = zeros(iH,1);             % upper bound AR pi 
vLowerUR = zeros(iH,1);             % lower bound AR ur
vUpperUR = zeros(iH,1);             % upper bound AR ur 
vFstat   = zeros(iH,1);             % F-stats 
vParB    = (-10:0.01:5)';           % parameters to test 
mWald    = zeros(size(vParB,1),iH); % store Wald
mTest    = zeros(size(vParB,1),iH); % store rejection 

% compute multiplier for h=0,1,2,...
for h=1:iH
    vPiC = vPi;   
    vUC  = vU ;
    if h>1                                       % construct cumulative pi and u 
        for j=2:h
            vPiC = vPiC + lagmatrix(vPi,-(j-1));
            vUC  = vUC  + lagmatrix(vU,-(j-1));
        end
    end 
    
    mObs     = [vPiC vUC mZ mX];                % drop the missing observations 
    mObs     = rmmissing(mObs);
    
    iTc      = size(mObs,1);                    % observations 
    
    vPiCu    = mObs(:,1);
    vUCu     = mObs(:,2);
    mZC      = mObs(:,3);
    mXC      = [ones(iTc,1) mObs(:,4:end)];    % controls PM     
   
    iL       = size(mZC , 2);                  % number of instruments   
    
    % COMPUTE THE PM  
    mMx      = eye(iTc) - mXC * inv(mXC'*mXC) * mXC';  % project out constant and control variables   
    vPiC     = mMx * vPiCu;
    vUC      = mMx * vUCu;    

    % IV reg 
    vCoefs   = inv(vUC' * mZC * inv(mZC'*mZC) * mZC' * vUC) * vUC' * mZC * inv(mZC'*mZC) * mZC' * vPiC; 
    vBeta(h) = vCoefs(1);                      % 2SLS coefs 
    
     % standard bounds 
    vRes   = vPiC -  vBeta(h) * vUC;
    mV     = NeweyWest(vRes, mZC ,-1,0);
    mPz    = mZC * inv(mZC' * mZC) * mZC';
    mAvar  = inv(vUC' * mPz * vUC) * vUC' * mZC * inv(mZC' * mZC) * mV * inv(mZC' * mZC) * mZC' * vUC * inv(vUC' * mPz * vUC);
    vLowerAR2s(h) = vBeta(h) - 1.96 * sqrt(mAvar);
    vUpperAR2s(h) = vBeta(h) + 1.96 * sqrt(mAvar);

    for k = 1:size(vParB,1) 
        vRes   = vPiC - vParB(k) * vUC;                       % residual 
        vGamma = mZC' * vRes;       
        vRes1  = vRes - mZC *  inv(mZC'*mZC) *  vGamma;       
        mV     = NeweyWest(vRes1, mZC ,4,0);                  % dimension 2L x 2L 
        
        mWald(k,h) = vGamma' * inv(mV) * vGamma;               % AR statistic  
      
        if (mWald(k,h) < chi2inv(0.95,iL))
            mTest(k,h) = 1; 
        end
        
    end

     % find upper and lower confidence bounds AR statistic  
    vBounds = FindUpperLower(mTest(:,h), vParB);
    vLowerAR(h) = vBounds(1);
    vUpperAR(h) = vBounds(2);    
   
    % compute effective F-stat 
    vFstat(h)  = EffectiveFstat(vUC, mZC);     
    
    % Compute IRF PI 
    vBetaPi(h) = inv(mZC'*mZC) * mZC' * vPiC;
    vRes       = vPiC - mZC *  vBetaPi(h);       
    mV         = NeweyWest(vRes, mZC ,4,0); 
    mAvar      =  inv(mZC'*mZC) * mV * inv(mZC'*mZC);
    vLowerPI(h)=  vBetaPi(h) - 1.96 * sqrt(mAvar);
    vUpperPI(h)=  vBetaPi(h) + 1.96 * sqrt(mAvar);
    
    % Compute IRF UR
    vBetaUr(h) = inv(mZC'*mZC) * mZC' * vUC;
    vRes       = vUC - mZC *  vBetaUr(h);       
    mV         = NeweyWest(vRes, mZC ,4,0); 
    mAvar      =  inv(mZC'*mZC) * mV * inv(mZC'*mZC);
    vLowerUR(h)=  vBetaUr(h) - 1.96 * sqrt(mAvar);
    vUpperUR(h)=  vBetaUr(h) + 1.96 * sqrt(mAvar);

end

% Store all Romer Romer results 
mRomerRomer = [vLowerAR vBeta vUpperAR vLowerPI vBetaPi vUpperPI vLowerUR vBetaUr vUpperUR vFstat vLowerAR2s vUpperAR2s]; 


% %  % SET UP High Frequency part 
mData    = xlsread('DataWeakIVhffinal.xlsx');
vPi      = mData(:,6);
vU       = mData(:,3);
mZ       = sum(mData(:,28:30),2);    % sum monthly HF shocks 
mXF      = mData(:,31:end);
mX       = [lagmatrix(vPi,1) lagmatrix(vPi,2) lagmatrix(vPi,3) lagmatrix(vPi,4) lagmatrix(vU,1) lagmatrix(vU,2) lagmatrix(vU,3) lagmatrix(vU,4)];
iH       = 21;

vBeta    = zeros(iH,1);             % storage 2SLS point 
vBetaPi  = zeros(iH,1);             % storage irf pi   
vBetaUr  = zeros(iH,1);             % storage irf ur        
vLowerAR = zeros(iH,1);             % lower AR-bound  
vUpperAR = zeros(iH,1);             % upper AR-bound
vLowerAR2s= zeros(iH,1);            % lower 2SLS-bound  
vUpperAR2s= zeros(iH,1);            % upper 2SLS-bound  
vLowerPI = zeros(iH,1);             % lower bound AR pi 
vUpperPI = zeros(iH,1);             % upper bound AR pi 
vLowerUR = zeros(iH,1);             % lower bound AR ur
vUpperUR = zeros(iH,1);             % upper bound AR ur 
vWaldMin = zeros(iH,1);             % minumum of wald region 
vFstat   = zeros(iH,1);             % F-stats 
vParB    = (-10:0.01:5)';           % parameters to test 
mWald    = zeros(size(vParB,1),iH); % store Wald
mTest    = zeros(size(vParB,1),iH); % store rejection 

% compute multiplier for h=0,1,2,...
for h=1:iH
    vPiC = vPi;   
    vUC  = vU ;
    if h>1                                       % construct cumulative pi and u 
        for j=2:h
            vPiC = vPiC + lagmatrix(vPi,-(j-1));
            vUC  = vUC  + lagmatrix(vU,-(j-1));
        end
    end 
    
    mObs     = [vPiC vUC mZ mX];                % drop the missing observations 
    mObs     = rmmissing(mObs);
    
    iTc      = size(mObs,1);                                     % observations 
   
    vPiCu    = mObs(:,1);
    vUCu     = mObs(:,2);
    mZC      = mObs(:,3);
    mXC      = [ones(iTc,1) mObs(:,4:end)];                        % controls PM     
  
    % COMPUTE THE PM  
    mMx      = eye(iTc) - mXC * inv(mXC'*mXC) * mXC';  % project out constant and control variables   
    
    vPiC     = mMx * vPiCu;
    vUC      = mMx * vUCu;    

    % IV reg 
    vCoefs   = inv(vUC' * mZC * inv(mZC'*mZC) * mZC' * vUC) * vUC' * mZC * inv(mZC'*mZC) * mZC' * vPiC; 
    vBeta(h) = vCoefs(1);                      % 2SLS coefs 
    
    
    % standard bounds 
    vRes   = vPiC -  vBeta(h) * vUC;
    mV     = NeweyWest(vRes, mZC ,12,0);
    mPz    = mZC * inv(mZC' * mZC) * mZC';
    mAvar  = inv(vUC' * mPz * vUC) * vUC' * mZC * inv(mZC' * mZC) * mV * inv(mZC' * mZC) * mZC' * vUC * inv(vUC' * mPz * vUC);
    vLowerAR2s(h) = vBeta(h) - 1.96 * sqrt(mAvar);
    vUpperAR2s(h) = vBeta(h) + 1.96 * sqrt(mAvar);
   
    
    % AR bounds 
    for k = 1:size(vParB,1) 
        vRes   = vPiC - vParB(k) * vUC;                       % residual 
        vGamma = mZC' * vRes;       
        vRes1  = vRes - mZC *  inv(mZC'*mZC) *  vGamma;       
        mV     = NeweyWest(vRes1, mZC ,-1,0);                  % dimension 2L x 2L 
        
        mWald(k,h) = vGamma' * inv(mV) * vGamma;               % AR statistic  
      
        if (mWald(k,h) < chi2inv(0.95,1))
            mTest(k,h) = 1; 
        end
        
    end

     % find upper and lower confidence bounds AR statistic  
    vBounds = FindUpperLower(mTest(:,h), vParB);
    vLowerAR(h) = vBounds(1);
    vUpperAR(h) = vBounds(2);    
   
    % compute effective F-stat 
    vFstat(h)  = EffectiveFstat(vUC, mZC);     
    
    mZC = sum(mZC,2);
    
    % Compute IRF PI 
    vBetaPi(h) = inv(mZC'*mZC) * mZC' * vPiC;
    vRes       = vPiC - mZC *  vBetaPi(h);       
    mV         = NeweyWest(vRes, mZC ,12,0); 
    mAvar      =  inv(mZC'*mZC) * mV * inv(mZC'*mZC);
    vLowerPI(h)=  vBetaPi(h) - 1.96 * sqrt(mAvar);
    vUpperPI(h)=  vBetaPi(h) + 1.96 * sqrt(mAvar);
    
    % Compute IRF UR
    vBetaUr(h) = inv(mZC'*mZC) * mZC' * vUC;
    vRes       = vUC - mZC *  vBetaUr(h);       
    mV         = NeweyWest(vRes, mZC ,12,0); 
    mAvar      =  inv(mZC'*mZC) * mV * inv(mZC'*mZC);
    vLowerUR(h)=  vBetaUr(h) - 1.96 * sqrt(mAvar);
    vUpperUR(h)=  vBetaUr(h) + 1.96 * sqrt(mAvar);
 
end

% Store all HF results
mHFI = [vLowerAR vBeta vUpperAR vLowerPI vBetaPi vUpperPI vLowerUR vBetaUr vUpperUR vFstat vLowerAR2s vUpperAR2s]; 


% MAKE FIGURE 
kappah= [NaN(5,1) ; mRomerRomer(6:end,2)];
kappahub=mRomerRomer(:,3);
kappahlb=mRomerRomer(:,1);

H=iH;

error1 = abs(mRomerRomer(:,2) - kappahlb);
error2 = abs(mRomerRomer(:,2) - kappahub);

x=(1:length(kappah))';

figure, 
sbp2=subplot(221), hold on
plot(x,0*kappah ,'LineWidth',1,'Color',[.7 .7 .7],'LineStyle','--');
H1= shadedErrorBar(x,mRomerRomer(:,2),[error2 error1], 'k');
g(1)=plot(kappah,'Linewidth',2,'Color','k','LineStyle','-');
g(1)=plot(mRomerRomer(:,11),'Linewidth',1,'Color','k','LineStyle',':');
g(1)=plot(mRomerRomer(:,12),'Linewidth',1,'Color','k','LineStyle',':');
ylabel('$\mathcal P_h$','Interpreter','latex','Fontsize',14);
ylim([-5 1])
xlim([1 H]);
set(gca','Xticklabel',num2str((0:5:H+1)'))
set(gca,'Xtick',1:5:H+1), xlim([1 H]),
set(gca,'Ytick',-14:2:14), xlim([1 H]),
ax=get(sbp2,'Position');
% ax(2)=ax(2)-.217;
% ax(4)=ax(4)+0.22; %or wathever
set(sbp2,'Position',ax);
title('Pre-1990, RR','Interpreter','latex')

kappah= [NaN(5,1) ; mHFI(6:end,2)];
kappahub= mHFI(:,3);
kappahlb= mHFI(:,1);
Fstat= mHFI(:,10);

H=iH;

error1 = abs(mHFI(:,2) - kappahlb);
error2 = abs(mHFI(:,2) - kappahub);

x=(1:length(kappah))';


sbp2=subplot(222), hold on
plot(x,0*kappah ,'LineWidth',1,'Color',[.7 .7 .7],'LineStyle','--');
H1= shadedErrorBar(x,mHFI(:,2),[error2 error1], 'k');
g(1)=plot([kappah(1:end)],'Linewidth',2,'Color','k','LineStyle','-');
g(1)=plot(mHFI(:,11),'Linewidth',1,'Color','k','LineStyle',':');
g(1)=plot(mHFI(:,12),'Linewidth',1,'Color','k','LineStyle',':');
rectangle('Position',[6 -0.6 H-6 0.9],'EdgeColor','r','LineStyle','--');
ylim([-5 1])
xlim([1 H]);
set(gca','Xticklabel',num2str((0:5:H+1)'))
set(gca,'Xtick',1:5:H+1), xlim([1 H]),
set(gca,'Ytick',-14:2:14), xlim([1 H]),

ax=get(sbp2,'Position');
set(sbp2,'Position',ax);
title('Post-1990, HFI','Interpreter','latex');



kappah=   mHFI(5:end,2);
kappahub= mHFI(5:end,3);
kappahlb= mHFI(5:end,1);

H=iH;

error1 = abs(mHFI(5:end,2) - kappahlb);
error2 = abs(mHFI(5:end,2) - kappahub);
vL2SLS = mHFI(5:end,11);
vU2SLS = mHFI(5:end,12);

x=(5:length(kappah)+4)';

sbp2=subplot(2,2,[3 4]), hold on
plot(x,0*kappah ,'LineWidth',1,'Color',[.7 .7 .7],'LineStyle','--');
H1= shadedErrorBar(x,mHFI(5:end,2),[error2 error1], 'k');
g(1)=plot(x,[kappah(1:end)],'Linewidth',2,'Color','k','LineStyle','-');
g(1)=plot(x, vL2SLS,'Linewidth',1,'Color','k','LineStyle',':');
g(1)=plot(x, vU2SLS,'Linewidth',1,'Color','k','LineStyle',':');
ylim([-0.6 0.2])
xlim([5 H]);
ylabel('$\mathcal P_h$','Interpreter','latex','Fontsize',14);
 set(gca,'Ytick',-14:.2:14), xlim([5 H]),

ax=get(sbp2,'Position');
% ax(2)=ax(2)-.217;
% ax(4)=ax(4)+0.22; %or wathever
set(sbp2,'Position',ax);
title('Post-1990, HFI','Interpreter','latex');

% PLOT IRF's 

%%%%%%%%%%%%%%%%%
% 1) Plot IRFs from  pre 1990
%%%%%%%%%%%%%%%%%

%rescale IRFs so that ir of ur peaks at 1:
ir   = mRomerRomer(:,[8 5]) / max(mRomerRomer(:,8));
irub = mRomerRomer(:,[9 6]) / max(mRomerRomer(:,8));
irlb = mRomerRomer(:,[7 4]) / max(mRomerRomer(:,8));

error1 = abs(ir - irlb);
error2 = abs(ir - irub);

H=length(ir);
x=(1:H)';


figure(2),
kk=0;

for ii=1:2:size(ir,2)*2
    subplot(2,2,ii), hold on
    kk=kk+1;
    plot(x,0*ir(:,kk) ,'LineWidth',1,'Color',[.7 .7 .7],'LineStyle','--');
    H1= shadedErrorBar(x,ir(:,kk),[error2(:,kk) error1(:,kk)], 'k');
    g(1)=plot(ir(:,kk),'Linewidth',2,'Color','k','LineStyle','-');
    if kk==1
        ylabel('{$\mathcal R^{\bar{u}}_h$}','Interpreter','latex','Fontsize',14);
    title('Pre-1990, RR','Interpreter','latex')  
    end
    if kk==2
        ylabel('{$\mathcal R^{\bar{\pi}}_h$}','Interpreter','latex','Fontsize',14);
    end
    xlim([1 H]);
    set(gca','Xticklabel',num2str((0:5:H)'))
    set(gca,'Xtick',1:5:H), xlim([1 H]),
    
end

%%%%%%%%%%%%%%%%%
% 2) Plot IRFs from HFI post 1990 (pre-ZLB)
%%%%%%%%%%%%%%%%%

%rescale IRFs so that ir of ur peaks at 1:

ir  = mHFI(:,[8 5]) / max(mHFI(:,8));
irub= mHFI(:,[9 6]) / max(mHFI(:,8));
irlb= mHFI(:,[7 4]) / max(mHFI(:,8));

error1 = abs(ir - irlb);
error2 = abs(ir - irub);

H=length(ir);
x=(1:H)';

figure(2),
kk=0;
for ii=2:2:size(ir,2)*2
    subplot(2,2,ii), hold on
    kk=kk+1;
    plot(x,0*ir(:,kk) ,'LineWidth',1,'Color',[.7 .7 .7],'LineStyle','--');
    H1= shadedErrorBar(x,ir(:,kk),[error2(:,kk) error1(:,kk)], 'k');
    g(1)=plot(ir(:,kk),'Linewidth',2,'Color','k','LineStyle','-');
    if kk==1
      title('Post-1990, HFI','Interpreter','latex')
    end
    set(gca','Xticklabel',num2str((0:5:H)'))
    set(gca,'Xtick',1:5:H), xlim([1 H]),
    
end



